/*
-----------------------------------------------------------------------------
This source file is part of OGRE
    (Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2005 The OGRE Team
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
LGPL like the rest of the engine.
-----------------------------------------------------------------------------
*/

/**
    \file 
        Lighting.h
    \brief
        Shows lighting support in OGRE. Also demonstrates the use of billboards
        and controllers for automatic time-relative behaviour.
*/


#include "ExampleApplication.h"
#include "OgreIlluminationManager.h"
#include "NXOgre.h"
#include "SceneSerializer.h"
#include "FPSPlayer.h"

AnimationState* mAnimState = 0;
bool cullCamera = true;
Body *throwSphere = 0;
Scene* NXScene = 0;
FPSPlayer* gFPSPlayer;

// Listener class for frame updates
class MoriaDemoFrameListener : public ExampleFrameListener
{
protected:
	
	unsigned long framecount;
	SceneNode* mainLightNode;
	Light* light;
	SceneNode* sysNode;
	
public:
    MoriaDemoFrameListener(RenderWindow* window, Camera* maincam, SceneNode* lightNode, Light* l, SceneNode* sysNode)
        :ExampleFrameListener(window, maincam) 
				 
    {
		mainLightNode = lightNode;
		light = l;
		framecount = 0;	
		mMoveSpeed /= 50.0;
		this->sysNode = sysNode;

    }

    bool frameStarted(const FrameEvent& evt)
    {	
		float thisTime = (float) Root::getSingleton().getTimer()->getMilliseconds() / 2000.0f;
		
		mainLightNode->setPosition(Vector3(2*cos(thisTime), 2, 2 * sin(thisTime)) + Vector3(5, 0, 5));
		Vector3 pos = light->getDerivedPosition();
		pos.normalise();
		pos *= -1;
		//light->setDirection( pos);
		
		framecount++;
	  
//	    mAnimState->addTime(evt.timeSinceLastFrame * 1.0);  
		

	//	sysNode->setPosition(mCamera->getPosition() - Vector3(0,1.5,0));

        return ExampleFrameListener::frameStarted(evt);     
    }

	bool processUnbufferedKeyInput(const FrameEvent& evt)
    {
		gFPSPlayer->processKeyInputs(mInputDevice);
/*
		static float lastTime = 0;
		static Timer* t = Root::getSingleton().getTimer();
		float thisTime = (float) t->getMilliseconds() / 1000.0f;
		float dt = thisTime-lastTime;
		lastTime = thisTime;
		
		
		if (mInputDevice->isKeyDown(KC_SPACE)) {
						
			if(throwSphere == 0)
			{
				throwSphere = NXScene->createBody("throwSphere",
													"sphere.mesh",
													new SphereShape(0.5f),
													Vector3(0,0,0),
													"mass: 10");
				throwSphere->getNode()->setScale(0.005,0.005,0.005);
			}
			//myCube->
			throwSphere->setGlobalPosition(mCamera->getPosition());
			throwSphere->addForce(mCamera->getDirection()*200);
	
		}*/

		
	//	return ExampleFrameListener::processUnbufferedKeyInput(evt);

		if( mInputDevice->isKeyDown( KC_ESCAPE) )
        {            
            return false;
        }
		if (mInputDevice->isKeyDown(KC_F) && mTimeUntilNextToggle <= 0)
        {
            mStatsOn = !mStatsOn;
            showDebugOverlay(mStatsOn);

            mTimeUntilNextToggle = 1;
        }
		if (mInputDevice->isKeyDown(KC_SYSRQ) && mTimeUntilNextToggle <= 0)
        {
			char tmp[20];
			sprintf(tmp, "screenshot_%d.png", ++mNumScreenShots);
            mWindow->writeContentsToFile(tmp);
            mTimeUntilNextToggle = 0.5;
			mWindow->setDebugText(String("Wrote ") + tmp);
        }
		return true;
	}

	bool processUnbufferedMouseInput(const FrameEvent& evt)
	{
		gFPSPlayer->processMouseInputs(mInputDevice);
//		return ExampleFrameListener::processUnbufferedMouseInput(evt);
		return true;
	}
};

/** Application class */
class MoriaDemoApplication : public ExampleApplication
{




protected:
	SceneNode* kupolaNode;
	SceneNode* mainLightNode;
	Light* mainLight;
	SceneNode *pSysNode;
	
	void loadResources(void)
	{
		// Initialise, parse scripts etc
		ResourceGroupManager::getSingleton().initialiseResourceGroup("Bootstrap");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_Basic");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("General");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_EnvMap");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("PostProc");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_Particles");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_PathMap");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_Moria");		
		//ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_Caustic");
	}

	void createPostproc()
	{		
		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GTP/Moria/PostProc/DoF");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GTP/Moria/PostProc/DoF", true);

	CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GTP/PostProc/Glow");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GTP/PostProc/Glow", true);

		
		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GTP/PostProc/ToneMap");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GTP/PostProc/ToneMap", true);	

		/*
		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GTP/PostProc/MotionBlur");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GTP/PostProc/MotionBlur", true);	*/
		
	}
    
	void chooseSceneManager()
	{
		mSceneMgr = mRoot->createSceneManager("OctreeSceneManager");
		//mSceneMgr = mRoot->createSceneManager("OcclusionCullingSceneManager");
	}

	void createScene(void) ///TOWERS
    {
		std::string occlusion_cfg("OcclusionCulling.cfg");
		//mSceneMgr->setWorldGeometry(occlusion_cfg);
		
		createPostproc();

		OgreIlluminationManager::getSingleton().setMainCamera(mCamera);
		OgreIlluminationManager::getSingleton().setMainViewport(mWindow->getViewport(0));
		 OgreIlluminationManager::getSingleton().setFocusingSM(false);
	   OgreIlluminationManager::getSingleton().setShadowMapSize(128);
	   OgreIlluminationManager::getSingleton().setBlurShadowMap(true);
	   OgreIlluminationManager::getSingleton().setShadowMapMaterialName(Light::LT_SPOTLIGHT, "GTP/Basic/Distance_Normalized");//GameTools/ShadowMapDistance
	   OgreIlluminationManager::getSingleton().setShadowMapMaterialName(Light::LT_POINT, "GTP/Basic/Distance_Normalized");
	   OgreIlluminationManager::getSingleton().setUseLISPSM(false);
	   OgreIlluminationManager::getSingleton().joinSharedRuns(false);
				
		Root::getSingleton()._setCurrentSceneManager(mSceneMgr);
		//mCamera->setPosition(-20,3,0);
		//mCamera->setPosition(0,100,0);
		//mCamera->lookAt(0,-1,0);
		mCamera->setFOVy(Radian(Degree(90)));
		mCamera->setNearClipDistance(0.001);
		mCamera->setFarClipDistance(150);
		// Set ambient light
        mSceneMgr->setAmbientLight(ColourValue(0.1, 0.1, 0.1));
		mSceneMgr->setSkyBox(true, "Examples/EveningSkyBox", 500);
		
		SceneNode* rootNode = mSceneMgr->getRootSceneNode();
		
		SceneSerializer s(mSceneMgr);
		DataStreamPtr inputStream; 				
		inputStream = ResourceGroupManager::getSingleton().openResource("moria.level");	//towers2.level	
		World* NXWorld = new World("FrameListener: no, log: html");
		NXWorld->getPhysXDriver()->createFrameListener(25);
		NXScene = NXWorld->createScene("Main", mSceneMgr, "gravity: yes, floor: no"); 
		s.setScene(NXScene);
		s.setWorld(NXWorld);
		s.parseScript(inputStream, ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
		s.parseEntryPoints("prmEntryPoints.text");

		NXScene->createActor("FakeFloor", new CubeShape(100,2,100), Vector3(0,-1,0), "static: yes"); 
		
/*
		Body* b = NXScene->createBody("ize",
							"sphere.mesh",
							new SphereShape(0.5f),
							Vector3(5,0,5),
							"mass: 10");
		b->getNode()->setScale(0.005,0.005,0.005);
		b->setGlobalPosition(Vector3(5,10,5));*/

/*
	   ParticleSystem* pSys1 = mSceneMgr->createParticleSystem("psys1", "GTP/Moria/Smoke_Large");
	   pSys1->setKeepParticlesInLocalSpace(false);
	   SceneNode *pSysNode = rootNode->createChildSceneNode("psys1");
	   pSysNode->attachObject(pSys1);
	   pSysNode->setPosition(-40,-1,-15);*/
		
	   /*Entity* object = mSceneMgr->createEntity("object", "head.mesh");
		object->setMaterialName("GTP/Caustic/Glass");
		SceneNode* objectNode = rootNode->createChildSceneNode();
		objectNode->attachObject(object);
		objectNode->scale(0.1,0.1,0.1);
		objectNode->rotate(Vector3(0,1,0),Radian(0.2));
		objectNode->setPosition(-30,2.0,-5);
		objectNode->_updateBounds();*/
			
		//createPlane("p","groundmaterial",Vector3(0,-0.3,0),Vector2(150,150));

		mainLight = mSceneMgr->createLight("MainLight");
		mainLight->setType(Light::LT_POINT);
		mainLight->setCastShadows(true);
        mainLight->setDiffuseColour(ColourValue::White);
		mainLight->setSpecularColour(ColourValue::White);
		mainLightNode = rootNode->createChildSceneNode();
		
		mainLightNode->setPosition(15,5,15);		
		mainLight->setPowerScale(100);
		//mainLight->setDirection(1,0, 0);
		mainLightNode->attachObject(mainLight);
		//mainLight->setSpotlightRange(Degree(0),Degree(120));
		mainLight->setAttenuation(200.0, 0, 0, 2);
/*
		Light* blueLight = mSceneMgr->createLight("blueLight");
		blueLight->setType(Light::LT_POINT);
		blueLight->setCastShadows(true);
		blueLight->setDiffuseColour(ColourValue(0.45,0.75,0.8));
		blueLight->setPowerScale(30);
		blueLight->setAttenuation(40.0, 0, 4, 0);
		SceneNode* blueLightNode = rootNode->createChildSceneNode();
		blueLightNode->setPosition(15,10,15);
		blueLightNode->attachObject(blueLight);*/
		
		Vector4 v = mainLight->getAs4DVector();

		BillboardSet* bbs = mSceneMgr->createBillboardSet("bb", 1);
		bbs->setDefaultDimensions(0.3, 0.3);
		bbs->createBillboard(Vector3::ZERO, ColourValue::White);
		bbs->setMaterialName("Flare");
		mainLightNode->attachObject(bbs);
/*
		    Entity* ent = mSceneMgr->createEntity("troll", "troll.mesh");
			ent->setCastShadows(true);
            // Add entity to the scene node
            SceneNode *trollNode = mSceneMgr->getRootSceneNode()->createChildSceneNode( Vector3(5,0,5));
			trollNode->attachObject(ent);
			trollNode->setScale(0.05,0.05,0.05);
            mAnimState = ent->getAnimationState("walk");
            mAnimState->setEnabled(true);*/

		PlayerCharacter::frameListenerPriority = 20;
		gFPSPlayer = new FPSPlayer(NXScene, mSceneMgr, Vector3(0.5, 1.8, 0.3), Vector3(5, 10, 15), 1.6, mCamera, false);
		//gFPSPlayer->addPlayerMesh("FPSarm.mesh", 0.02, true);
		//gFPSPlayer->addWeaponModel("sword.mesh", "joint28");

		PlayerCharacter* troll = new PlayerCharacter("troll1", NXScene, mSceneMgr, Vector3(1.5, 3.0, 1.2), Vector3(5, 10, 5), false);
		troll->addPlayerMesh("troll.mesh", 0.05, false);

		for(float i = 0; i< 7; i++)
		{
			float size = 0.5;
			ActorParams ac;
			ac.setToDefault();
			ac.mass = 1.0;
			ac.nodescale = size;
			Body* b = NXScene->createBody("cube.mesh",
								new CubeShape(size, size, size),
								Vector3(3,size/2.0 + i * 5.0,3),
								ac);
			b->setCharacterCollisionGroup(NX_CHARACTER_COL_GROUP_PUSHABLE);
		}
	
	  /* ParticleSystem* pSys1 = mSceneMgr->createParticleSystem("psys1", "MoriaDust");
	   pSys1->setKeepParticlesInLocalSpace(true);
	   pSysNode = rootNode->createChildSceneNode("psys1");
	   pSysNode->attachObject(pSys1);
	   pSysNode->setPosition(0,0,0);*/


		OgreIlluminationManager::getSingleton().initTechniques();
   }
	
	void createPlane(Ogre::String _name, Ogre::String _material, Ogre::Vector3 _pos, Ogre::Vector2 _size, Ogre::Vector2 _subdivisions = Ogre::Vector2(1,1)) 
	{
		Plane _plane;
        _plane.normal = Vector3::UNIT_Y;
        _plane.d = 0;


        MeshManager::getSingleton().createPlane(_name + ".plane",
            ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, 
            _plane,_size.x,_size.y,1,1,true,1,_subdivisions.x,_subdivisions.y,Vector3::UNIT_Z);
        
		Entity *_entity = mSceneMgr->createEntity(_name + ".entity", _name + ".plane");
        _entity->setMaterialName(_material);

		SceneNode* _node = mSceneMgr->getRootSceneNode()->createChildSceneNode(_name + ".node");
		_node->attachObject(_entity);
		_node->setPosition(_pos);

	}

	void createFrameListener(void)
    {
		// This is where we instantiate our own frame listener
		mFrameListener= new MoriaDemoFrameListener(mWindow,
			mCamera, mainLightNode, mainLight, pSysNode	);
        mFrameListener->setPriority(10);	
		mRoot->addFrameListener(mFrameListener);
//		gFPSPlayer->setPriority(20);
//		mRoot->addFrameListener(gFPSPlayer);
		OgreIlluminationManager::getSingleton().setPriority(40);
	    mRoot->addFrameListener(&OgreIlluminationManager::getSingleton());
	  
    }

};

